// Pseudo 'C' code to emulate an Armor Attack sound board.
//
// The routine 'play( WaveForm.wav)' is used to play a .WAV file from
// it's start.  If the file is already being played, 'play()' will
// stop the current playing of the file, and restart it from the begining.
//
// The routine 'loop( WaveForm.wav)' is used to play a .WAV file continuously
// by looping the file from end back to start.  This will be done until the
// routine 'stop( WaveForm.wav)' is called.
//
// The routines: play(), loop() and stop() are to be defined by you.

typedef unsigned char	uchar;
typedef unsigned int		uint;

extern void play( char *);			// play a wave file from the begining
extern void loop( char *);			// loop a wave file
extern void stop( char *);			// stop the looping of a wave file

#define	OTB_DATA			0x80
#define	OTB_CLOCK		0x10
#define	OTB_CHOPPER		0x08
#define	OTB_BEEP			0x04
#define	OTB_TANK			0x02
#define	OTB_SRLATCH		0x01

#define	SRB_TANKFIRE	0x80
#define	SRB_HIEXP		0x40
#define	SRB_JEEPFIRE	0x20
#define	SRB_LOEXP		0x10
#define	SRB_SPEED		0x0F

uchar	OldOutReg;							// keep a copy of previous OUT value
uchar	SRreg, OldSRreg;					// keep a copy of shift register and previous value
uchar	TSreg;								// Temporary shift register

// Called to initialize the sound emulation.

void initSounds( void)
{
	OldOutReg = OTB_CHOPPER | OTB_BEEP | OTB_TANK;
	SRreg = SRB_TANKFIRE | SRB_HIEXP | SRB_JEEPFIRE | SRB_LOEXP;
	OldSRreg = SRreg;
}

// This routine is to be called each time the CCPU's OUT register's
// bits 7, 4, 3, 2, 1 or 0 change states.
//
// outReg = New value of the CCPU's OUT register.

void updateSounds( uchar outReg)
{
	uchar outDiff;							// changed bits
	uchar	outLow;							// changed bits that have just gone low
	uchar	outHigh;							// changed bits that have just gone high

	uchar	srDiff;							// changed bits	
	uchar	srLow;							// changed bits that have just gone low

	outDiff = outReg ^ OldOutReg;		// get bits that have changed state
	outLow = outDiff & ~outReg;		// get bits that have just gone low
	outHigh = outDiff & outReg;		// get bits that have just gone high

	// check if TSR is being clocked

	if (outHigh & OTB_CLOCK)
	{	TSreg >>= 1;						// shift TSreg right one
		TSreg |= outReg & OTB_DATA;	// shift in new bit
	}

	// check if SR is being latched

	if (outHigh & OTB_SRLATCH)
	{	SRreg = TSreg;										// save new SR value

		// check for new SR sounds

		srDiff = SRreg ^ OldSRreg;						// get diff's
		srLow = srDiff & ~SRreg;						// get bits that just went low

		if (srLow & SRB_TANKFIRE)
			play( "TankFire.wav");

		if (srLow & SRB_HIEXP)
			play( "HiExp.wav");

		if (srLow & SRB_JEEPFIRE)
			play( "JeepFire.wav");

		if (srLow & SRB_LOEXP)
			play( "LoExp.wav");

		OldSRreg = SRreg;
	}

	// check CCPU OUT register sounds

	if (outLow & OTB_CHOPPER)
		loop( "Chopper.wav");

	if (outHigh & OTB_CHOPPER)
		stop( "Chopper.wav");

	if (outLow & OTB_BEEP)
		loop( "Beep.wav");

	if (outHigh & OTB_BEEP)
		stop( "Beep.wav");

	// Check to see if TANK sound started, or TANK speed is changed while TANK
	// sounds are being played.

	if (outLow & OTB_TANK || (!(outReg & OTB_TANK) && (srDiff & SRB_SPEED)))
	{
		stop( "Tank??.wav");				// stop any previous tank sounds

		// get speed of tank

		switch (SRreg & SRB_SPEED)
		{
		case 0:
			loop( "Tank00.wav");
			break;

		case 1:
			loop( "Tank01.wav");
			break;

			// etc...

		}
	}

	if (outHigh & OTB_TANK)
		stop( "TankX.wav");			// stop any tank sound

	OldOutReg = outReg;				// save new OUT register
}
