// Pseudo 'C' code to emulate a Rip Off sound board.
//
// The routine 'play( WaveForm.wav)' is used to play a .WAV file from
// it's start.  If the file is already being played, 'play()' will
// stop the current playing of the file, and restart it from the begining.
//
// The routine 'loop( WaveForm.wav)' is used to play a .WAV file continuously
// by looping the file from end back to start.  This will be done until the
// routine 'stop( WaveForm.wav)' is called.
//
// The routines: play(), loop() and stop() are to be defined by you.

typedef unsigned char	uchar;
typedef unsigned int		uint;

extern void play( char *);			// play a wave file from the begining
extern void loop( char *);			// loop a wave file
extern void stop( char *);			// stop the looping of a wave file

#define	OTB_EXPLOSION	0x80
#define	OTB_LASER		0x10
#define	OTB_TORPEDO		0x08
#define	OTB_SRLATCH		0x04
#define	OTB_CLOCK		0x02
#define	OTB_DATA			0x01

#define	SRB_SPEED		0x38
#define	SRB_BACKGROUND	0x04
#define	SRB_BEEP			0x02
#define	SRB_MOTOR		0x01

uchar	OldOutReg;							// keep a copy of previous OUT value
uchar	SRreg, OldSRreg;					// keep a copy of shift register and previous value
uchar	TSreg;								// Temporary shift register

// Called to initialize the sound emulation.

void initSounds( void)
{
	OldOutReg = OTB_EXPLOSION | OTB_LASER | OTB_TORPEDO;
	SRreg = SRB_BACKGROUND | SRB_BEEP | SRB_MOTOR;
	OldSRreg = SRreg;
}

// This routine is to be called each time the CCPU's OUT register's
// bits 7, 4, 3, 2, 1 or 0 change states.
//
// outReg = New value of the CCPU's OUT register.

void updateSounds( uchar outReg)
{
	uchar outDiff;							// changed bits
	uchar	outLow;							// changed bits that have just gone low
	uchar	outHigh;							// changed bits that have just gone high

	uchar	srDiff;							// changed bits	
	uchar	srLow;							// changed bits that have just gone low
	uchar	srHigh;							// changed bits that have just gone high

	outDiff = outReg ^ OldOutReg;		// get bits that have changed state
	outLow = outDiff & ~outReg;		// get bits that have just gone low
	outHigh = outDiff & outReg;		// get bits that have just gone high

	// check if TSR is being clocked

	if (outHigh & OTB_CLOCK)
	{	TSreg >>= 1;								// shift TSreg right one
		TSreg |= (outReg & OTB_DATA) << 7;	// add new bit
	}

	// check if SR is being latched

	if (outHigh & OTB_SRLATCH)
	{	SRreg = TSreg;										// save new SR value

		// check for new SR sounds

		srDiff = SRreg ^ OldSRreg;						// get diff's
		srLow = srDiff & ~SRreg;						// get bits that just went low
		srHigh = srDiff & SRreg;						// get bits that just went high

		if (srLow & SRB_MOTOR)
			loop( "Motor.wav");

		if (srHigh & SRB_MOTOR)
			stop( "Motor.wav");

		if (srLow & SRB_BEEP)
			loop( "Beep.wav");

		if (srHigh & SRB_BEEP)
			stop( "Beep.wav");

		// Check to see if Background sound started, or background speed is changed while
		// background sounds are being played.

		if (outLow & SRB_BACKGROUND || (!(SRreg & SRB_BACKGROUND) && (srDiff & SRB_SPEED)))
		{
			stop( "BackG?.wav");						// stop any previous background sounds

			// set speed of background

			switch (SRreg & SRB_SPEED)
			{
			case 0:
				loop( "BackG0.wav");
				break;

			case 1:
				loop( "BackG1.wav");
				break;

				// etc...

			}
		}

		if (srHigh & SRB_BACKGROUND)
			stop( "BackG?.wav");						// stop any background sounds

		OldSRreg = SRreg;
	}

	// check CCPU OUT register sounds

	if (outLow & OTB_EXPLOSION)
		play( "Explosion.wav");

	if (outLow & OTB_LASER)
		play( "Laser.wav");

	if (outLow & OTB_TORPEDO)
		play( "Torpedo.wav");

	OldOutReg = outReg;				// save new OUT register
}
