// Pseudo 'C' code to emulate Tailgunner sound board.
//
// The routine 'play( WaveForm.wav)' is used to play a .WAV file from
// it's start.  If the file is already being played, 'play()' will
// stop the current playing of the file, and restart it from the begining.
//
// The routine 'loop( WaveForm.wav)' is used to play a .WAV file continuously
// by looping the file from end back to start.  This will be done until the
// routine 'stop( WaveForm.wav)' is called.
//
// The routines: play(), loop() and stop() are to be defined by you.

typedef unsigned char	uchar;
typedef unsigned int		uint;

extern void play( char *);			// play a wave file from the begining
extern void loop( char *);			// loop a wave file
extern void stop( char *);			// stop the looping of a wave file

#define	OTB_STROBE		0x10
#define	OTB_DATA			0x08
#define	OTB_ADDRESS		0x07

#define	MUX_HYPERSPACE	0x20
#define	MUX_BOUNCE		0x10
#define	MUX_SHIELD		0x08
#define	MUX_LASER		0x04
#define	MUX_RUMBLE		0x02
#define	MUX_EXPLOSION	0x01

uchar	OldOutReg;							// keep a copy of previous OUT value
uchar	XRreg, OldXRreg;					// keep a copy of MUX register and previous value

// Called to initialize the sound emulation.
//
// outReg = New value of the CCPU's OUT register.

void initSounds( uchar outReg)
{
	XRreg = MUX_HYPERSPACE | MUX_RUMBLE;

	OldOutReg = outReg;					// synchronize the register variables
	OldXRreg = XRreg;
}

// This routine is to be called each time the CCPU's OUT register's
// bits 7, 4, 3, 2, 1 or 0 change states.
//
// outReg = New value of the CCPU's OUT register.

void updateSounds( uchar outReg)
{
	uchar outDiff;							// changed bits
	uchar	outLow;							// changed bits that have just gone low

	uchar	xrDiff;							// changed bits
	uchar xrHigh;							// changed bits that have just gone high
	uchar	xrLow;							// changed bits that have just gone low

	uchar	mask;								// bit mask

	outDiff = outReg ^ OldOutReg;		// get bits that have changed state
	outLow = outDiff & ~outReg;		// get bits that have just gone low

	if (outLow & OTB_STROBE)
	{	mask = 0x01 << (outReg & OTB_ADDRESS);		// get address of bit as a mask

		if (outReg & OTB_DATA)
			XRreg |= mask;									// if DATA set, set bit

		else
			XRreg &= ~mask;								// else reset bit

		// check for new MUX sounds

		xrDiff = XRreg ^ OldXRreg;						// get diff's
		xrLow = xrDiff & ~XRreg;						// get bits that just went low
		xrHigh = xrDiff & XRreg;						// get bits that just went high

		if (xrLow & MUX_HYPERSPACE)
			play( "Hyperspace.wav");

		if (xrLow & MUX_BOUNCE)
			play( "Bounce.wav");

		if (xrLow & MUX_SHIELD)
			loop( "Shield.wav");

		if (xrHigh & MUX_SHIELD)
			stop( "Shield.wav");

		if (xrLow & MUX_LASER)
			play( "Laser.wav");

		if (xrHigh & MUX_LASER)
			stop( "Laser.wav");

		if (xrLow & MUX_RUMBLE)
			loop( "Rumble.wav");

		if (xrHigh & MUX_RUMBLE)
			stop( "Rumble.wav");

		if (xrLow & MUX_EXPLOSION)
			play( "Explosion.wav");

		OldXRreg = XRreg;
	}
	OldOutReg = outReg;				// save new OUT register
}
