;----------------------------------------------------------------------------;
; BCMACROS.INC								     ;
;									     ;
; Macros needed to link assembler files to "C" code.			     ;
;									     ;
; (c) Copyright 1996, Zonn Moore.  All rights reserved.			     ;
;									     ;
;----------------------------------------------------------------------------;
		%NOLIST
		%NOCREF	t,s,c,m,l,h,x	;; don't cross reference these

		LOCALS		; all arguments are local in scope

; Setup library size using the command line define '_msize'

 ifndef _msize
		err	"_MSIZE not defined, assembly terminated."
		end
 endif

; Models.
;
; The assembler variable '_msize' is used to determine the model size being
; assembled.  It is set to one of the follow sizes:
;
;    _msize=t    ; Setup for a TINY model (not support by Microsoft "C")
;    _msize=s    ; Setup for a SMALL model
;    _msize=c    ; Setup for a COMPACT model
;    _msize=m    ; Setup for a MEDIUM model
;    _msize=l    ; Setup for a LARGE model
;    _msize=h    ; Setup for a HUGE model
;    _msize=x    ; Setup for a EXTENDED DOS model
;
; The variable '_msize' can be set in the source file, or from the
; assembler's command line using the syntax:
;
;    TMASM /D_msize=S [flags etc filename]
;
; The above example would create an SMALL model object file.

t		equ	0	; Tiny
s		equ	1	; Small
c		equ	2	; Compact
m		equ	3	; Medium
l		equ	4	; Large
h		equ	5	; Huge
x		equ	6	; Extended

 if     (_msize EQ t)
		model	tiny

 elseif (_msize EQ s)
		model 	small

 elseif (_msize EQ m)
		model	medium

 elseif (_msize EQ c)
		model	compact

 elseif (_msize EQ l)
		model	large

 elseif (_msize EQ h)
		model	huge

 elseif (_msize EQ x)
		model	large
 endif

; Some common ASCII equates.

CR		equ	<00Dh>		; Return
LF		equ	<00Ah>		; Line Feed
EOS		equ	<000h>		; End of string

;----------------------------------------------------------------------------;
; DUSES regs								     ;
;									     ;
; Sets up a register 'uses' statement if the current model DATA is a	     ;
; large pointer.							     ;
;									     ;
; Call with:								     ;
;    regs = String of registers (if more than 1 regs, string must be	     ;
;           inclosed in '<>'s).						     ;
;----------------------------------------------------------------------------;
macro		DUSES	regs

 if @DataSize
		uses	regs
 endif
endm

;----------------------------------------------------------------------------;
; LDDS reg,adr								     ;
;									     ;
; Load a Data pointer into DS:reg.					     ;
; If small Data model, then DS is already loaded.			     ;
;									     ;
; Call:   reg = Offset register to be loaded.				     ;
;         adr = Address of data to load.				     ;
;----------------------------------------------------------------------------;
macro		LDDS	reg,adr

 if @DataSize
		lds	reg,adr
 else
		mov	reg,adr
 endif
endm	

;----------------------------------------------------------------------------;
; LDES reg,adr								     ;
;									     ;
; Load a Data pointer into ES:reg.					     ;
;									     ;
; If small Data model, then moves DS into ES.				     ;
;									     ;
; Call:   reg = Offset register to be loaded.				     ;
;         adr = Address of data to load.				     ;
;----------------------------------------------------------------------------;
macro		LDES	reg,adr

 if @DataSize
		les	reg,adr
 else
		mov	reg,ds
		mov	es,reg
		mov	reg,adr
 endif
endm	

;----------------------------------------------------------------------------;
; LCDS reg,adr								     ;
;									     ;
; Load a Code pointer into DS:reg.					     ;
;									     ;
; If small Code model, then DS is loaded with CS.			     ;
;----------------------------------------------------------------------------;
macro		LCDS	reg,adr

 if @CodeSize
		lds	reg,adr
 else
		mov	reg,cs
		mov	ds,reg
		mov	reg,adr
 endif
endm	

;----------------------------------------------------------------------------;
; LCES reg,adr								     ;
;									     ;
; Load a Code pointer into ES:reg.					     ;
;									     ;
; If small Code model, then ES is loaded with CS.			     ;
;----------------------------------------------------------------------------;
macro		LCES	reg,adr

 if @CodeSize
		les	reg,adr
 else
		mov	reg,cs
		mov	es,reg
		mov	reg,adr
 endif
endm	

;----------------------------------------------------------------------------;
; DNULL seg,reg								     ;
;									     ;
; Test if seg:reg is a NULL Data Ptr.					     ;
;									     ;
;    seg = Segment register of pointer to test.				     ;
;    reg = Offset register of pointer to test.				     ;
;----------------------------------------------------------------------------;
macro		DNULL	seg,reg

 if @DataSize
  ifdifi <reg>,<AX>
		push	ax
		mov	ax,seg
		or	ax,reg
		pop	ax
  else
		push	bx
		mov	bx,seg
		or	bx,reg
		pop	bx
  endif
 else
		test	reg,reg
 endif
endm	

;----------------------------------------------------------------------------;
; CNULL seg,reg								     ;
;									     ;
; Test if seg:reg is a NULL Code Ptr.					     ;
;									     ;
;    seg = Segment register of pointer to test.				     ;
;    reg = Offset register of pointer to test.				     ;
;----------------------------------------------------------------------------;
macro		CNULL	 seg,reg

 if @CodeSize
  ifdifi <reg>,<AX>
		push	ax
		mov	ax,seg
		or	ax,reg
		pop	ax
  else
		push	bx
		mov	bx,seg
		or	bx,reg
		pop	bx
  endif
 else
		test	reg,reg
 endif
endm

;----------------------------------------------------------------------------;
; IOWAIT								     ;
;									     ;
; Macro for wasting time after an I/O access.				     ;
;----------------------------------------------------------------------------;
macro		IOWAIT
		local	carry,nocarry
		%NOCREF	carry,nocarry
		jc	carry	;; give the appearence of real code to any
		jnc	nocarry	;; intelligent instruction chacheing processor.
carry:		stc
nocarry:
endm	

		%LIST
