;----------------------------------------------------------------------------;
; TIMERMS.ASM                                                                ;
;                                                                            ;
; (c) Copyright 1999, Zonn Moore.  All rights reserved.                      ;
;----------------------------------------------------------------------------;
		Ideal
      		include	"bcmacros.inc"

; Point to BIOS data.

segment		ABS0	at 0
		org	00400h+0006Ch		; Address of BIOS timer
timer_low	dw	?			; BIOS's tick timer
		ends

		CodeSeg	MISCC

;----------------------------------------------------------------------------;
; int gettimer_qk()							     ;
;									     ;
; Returns:								     ;
;   Short integer of 8253 timer count.					     ;
;----------------------------------------------------------------------------;
public		_gettimer_qk
proc		_gettimer_qk	c

		push	ds
		sub	ax,ax
		mov	ds,ax
		assume	ds:ABS0
		sub	al,al
		out	043h,al		; latch timer 0
		IOWAIT			; wait for I/O
		in	al,040h		; get timer
		xchg	al,ah
		in	al,040h
		xchg	al,ah		; else, fix counter
		neg	ax		; make count apppear to count up
		pop	ds
		ret

		endp

;----------------------------------------------------------------------------;
; int testtimer_qk( uint timer, uint count))				     ;
;									     ;
; Test if the number of 8253 ticks in 'count' has passed.		     ;
;									     ;
; Returns:								     ;
;    0 - if timer not timed out.					     ;
;    1 - if timer timed out.						     ;
;----------------------------------------------------------------------------;
public		_testtimer_qk
proc		_testtimer_qk	c

		arg	timer:Dword,count:Word

		call	_gettimer_qk
		sub	ax,[Word timer]		; get ticks that have passed
		cmp	ax,[Word count]		; are ticks greater than test value?
		mov	ax,0			; (zero AX)
		jc	tqk1			; if not timed out, jump
		inc	ax			; else, set timeout flag
tqk1:		ret
		endp

;----------------------------------------------------------------------------;
; void waittimer_qk( uint count)					     ;
;									     ;
; Wait for the number of 8253 ticks in 'count' to pass.			     ;
;----------------------------------------------------------------------------;
public		_waittimer_qk
proc		_waittimer_qk	c

		arg	count:Word

		call	_gettimer_qk		; get current count
		mov	bx,ax			; save it
		mov	cx,[Word count]		; get count

wqk1:		call	_gettimer_qk		; get current count
		sub	ax,bx			; get ticks that have passed
		cmp	ax,cx			; are ticks greater than test value?
		jc	wqk1			; no, loop
		ret
		endp

;----------------------------------------------------------------------------;
; (long)gettimer_ms()							     ;
;									     ;
; Returns:								     ;
;   Long integer of timer count.					     ;
;----------------------------------------------------------------------------;
public		_gettimer_ms
proc		_gettimer_ms	c

		call	getdc		; get double count
		ret
		endp

;----------------------------------------------------------------------------;
; (int)testtimer_ms( timer, count)					     ;
;									     ;
; Test if the number of milliseconds in 'count' has passed.		     ;
;									     ;
; Returns:								     ;
;    0 - if timer not timed out.					     ;
;    1 - if timer timed out.						     ;
;----------------------------------------------------------------------------;
public		_testtimer_ms
proc		_testtimer_ms	c

		arg	timer:Dword,count:Word

		mov	ax,1193			; number of ticks in a millisecond
		mul	[count]			; get count in milliseconds
		mov	cx,ax			; place in BX:CX
		mov	bx,dx
		
		mov	ax,[Word timer]		; get timer
		mov	dx,[Word timer+2]

		call	tstdc			; test double count
		mov	ax,0
		jc	ttm1			; if not timed out, jump
		inc	ax			; else, set timeout flag
ttm1:		ret
		endp

;----------------------------------------------------------------------------;
; int testtimer_tk( timer, count))					     ;
;									     ;
; Test if the number of 8253 ticks in 'count' has passed.		     ;
;									     ;
; Returns:								     ;
;    0 - if timer not timed out.					     ;
;    1 - if timer timed out.						     ;
;----------------------------------------------------------------------------;
public		_testtimer_tk
proc		_testtimer_tk	c

		arg	timer:Dword,count:Word

		mov	cx,[Word count]		; get count in BX:CX
		mov	bx,[Word count+2]
		mov	ax,[Word timer]		; get timer
		mov	dx,[Word timer+2]
		call	tstdc			; test double count
		mov	ax,0
		jc	ttk1			; if not timed out, jump
		inc	ax			; else, set timeout flag
ttk1:		ret
		endp

;----------------------------------------------------------------------------;
; Test if the number of ticks in BX:CX has passed since the count in DX:AX.  ;
;									     ;
; Called:								     ;
;    DX:AX = Double word timer of start time.				     ;
;    BX:CX = Double word count of time to test.				     ;
;									     ;
; Returns with:								     ;
;    C-flg = Reset when timer times out.				     ;
;            Set if timer not timed out.				     ;
;									     ;
; Uses:									     ;
;    CC									     ;
;----------------------------------------------------------------------------;
proc		tstdc		near

		push	bp
		push	bx
		push	cx
		push	dx		; place old timer on stack
		push	ax
		mov	bp,sp		; point to stack
		call	getdc		; get double count
		sub	ax,[bp]		; subtract old timer
		sbb	dx,[bp+2]
		cmp	dx,bx		; see if range large enough
		jne	tstd1
		cmp	ax,cx
tstd1:		pop	cx		; carry flag reflects results
		pop	bx
		pop	ax
		pop	dx
		pop	bp
		ret
		endp

;----------------------------------------------------------------------------;
; Get a double word count from BIOS's low timer word and the 8253.	     ;
;									     ;
; 8253 must be setup in mode 2 for this routine to be meaningful.	     ;
;									     ;
; Returns:								     ;
;    DX:AX = Double word count.						     ;
;									     ;
; Uses:									     ;
;    CC, AX, DX								     ;
;----------------------------------------------------------------------------;
proc		getdc		near

		push	ds
		sub	ax,ax
		mov	ds,ax
		assume	ds:ABS0
gdc1:		mov	dx,[timer_low]	; get BIOS low timer tick
		sub	al,al
		out	043h,al		; latch timer 0
		IOWAIT			; wait for I/O
		in	al,040h		; get timer
		xchg	al,ah
		in	al,040h
		cmp	dx,[timer_low]	; has timer overflowed?
		jne	gdc1		; yes, try again
		xchg	al,ah		; else, fix counter
		neg	ax		; make count apppear to count up
		pop	ds
		ret
		endp

		end
